#!/usr/bin/perl

# Linktausch-Check
# Copyright (C) 2002-2006 Robert Boeck
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


require LWP::UserAgent;
require HTTP::Request;


# ----- user defined variables -----

$proxy = '';                               # http proxy
$useragent = 'Linktausch-Check v0.4 (http://robert.boeck-horn.de/ltcheck.html)';       # user agent
$referer = '-';                            # referer

# ----- end of user defined variables -----

$urlfile = '';

@links = ();
%disallow = ();

&getparams;

&get_data;


$ua = LWP::UserAgent->new;
$ua->proxy("http", $proxy) if ($proxy);
$ua->agent($useragent) if ($useragent);

foreach $record (@links) {
  ($source, @dests) = split (/\s+/, $record);

  %disallow = ();

  if ($source =~ /^http:\/\//i) {

    &get_robots_txt ($source);

    print "Source: retrieving $source ... ";

    $request = HTTP::Request->new('GET', $source);
    $request->header(Referer => $referer);
    $response = $ua->request($request);
    if ($response->is_success) {
      $page = $response->content;

      print "done.";

      $page =~ s/(\n|\r)/ /g;
      $page =~ s/<!--.*?-->/ /g;

      $links = $page =~ s/(<a[^>]+href\s*=\s*[^>]+>)/$1/ig;

      print " ($links links)\n";

      foreach $dis (keys %disallow) {
        if ($source =~ /http:\/\Q$dis\E/) {
          print " ??? $dis disabled for $disallow{$dis} in robots.txt\n";
        }
      }

      foreach $dest (@dests) {
        $found = 0;
        $dest1 = &domain ($dest);
        while ($page =~ /<a([^>]+href\s*=\s*[^>]+)>(.*?)<\/a>/gi) {
          $a_par = $1;
          $a_txt = $2;
          $linktarget = &getparam ($a_par, 'href');
          if ($linktarget =~ /\Q$dest1\E/i) {
            $found++;
            $linktitle = &getparam ($a_par, 'title');
            $linktitle = "\n     title: ".$linktitle if ($linktitle);
            $linkrel = &getparam ($a_par, 'rel');
            if (lc ($linkrel) eq 'nofollow') {
              $linkrel = "\n !!! rel: ".$linkrel;
            }
            elsif ($kinkrel) {
              $linkrel = "\n     rel: ".$linkrel;
            }
            $linktext = &get_linktext ($a_txt);
            $linktext = "\n     ".$linktext if ($linktext);
            if ($linktarget =~ /^\Q$dest\E$/) {
              print " --> $dest ... exact match ($found)$linktitle$linkrel$linktext\n";
            }          
            else {
              print " --> $dest1 ... loose match ($found): $linktarget$linktitle$linkrel$linktext\n";
            }
          }
        }
        unless ($found) {
          print " !!! $dest ... not found\n";
        }
      }

      print "\n";
    }
    else {
      print "failed.\n";
    }

  }
  else {
    print "$source ist not a valid URL\n";
  }

}


sub get_robots_txt {
  my $url = shift;
  $url =~ s/(http:\/\/[^\/]+).*/$1\/robots.txt/;
  my $req = HTTP::Request->new('GET', $url);
  $req->header(Referer => $referer);
  my $res = $ua->request($req);
  if ($res->is_success) {
    my $robots_txt = $res->content;
    my @robots_txt = split (/\n/, $robots_txt);
    chomp (@robots_txt);
    my $bot = '';
    foreach my $line (@robots_txt) {
      if ($line =~ /user-agent\s*:\s*([^#]*)/i) {
        $bot = $1;
        $bot = &trim ($bot);
      }
      elsif ($line =~ /disallow\s*:\s*(\S*)/i) {
        if ($1) {
          $disallow{$1} .= ',' if ($disallow{$1});
          $disallow{$1} .= $bot;
        }
      }
    }
  }
}

sub get_data {
  open (DATA, $urlfile) or die ("cannot read $urlfile\n");
  while (my $line = <DATA>) {
    chomp ($line);
    if ($line !~ /^\s*(#|;)/) {
      $line = &trim ($line);
      push (@links, $line) if ($line);
    }
  }
  close (DATA);
}

sub getparams {
  foreach my $arg (@ARGV) {
    if ($arg =~ /^\-/) {
      if ($arg =~ /^\-h$/i) {
        die "usage: ltcheck.pl <urlfile>\n\n";
      }
      else { die "unknown parameter: $arg\n\n"; }
    }
    elsif (! $urlfile) { $urlfile = $arg; }
    else { die "unknown parameter: $arg\n"; }
  }
  unless ($urlfile) {
    $urlfile = 'ltcheck.txt';
    unless (-e $urlfile) {
      $urlfile = $0;
      $urlfile =~ s/(\/|\\)[^\/\\]*$/$1/;
      $urlfile .= 'ltcheck.txt';
    }
    unless (-e $urlfile) {
      die "usage: ltcheck.pl <urlfile>\n\n";
    }
  }
}

sub getparam {
  my ($result, $parn) = @_;
  my @params = ();
  push (@params, $+) while $result =~ /\s*([^"=]+=\s*"[^"]*")|\s*([^'=]+=\s*'[^']*')|\s*([^=]+=\s*[^\s]*)\s*/g;
  $result = '';
  foreach my $param (@params) {
    if ($param =~ /^$parn\s*=\s*['"]?(.*?)['"]?$/i) {
      $result = $1;
      last;
    }
  }
  return $result;
}

sub get_linktext {
  my $linktext = shift;
  if ($linktext =~ /^\s*<img([^>]+)>\s*$/i) {
    my $imgsrc = &getparam ($1, 'src');
    my $imgalt = &getparam ($1, 'alt');
    my $imgtitle = &getparam ($1, 'title');
    $linktext  = 'link-img: '.$imgsrc;
    $linktext .= ', alt: '.$imgalt if ($imgalt);
    $linktext .= ', title: '.$imgtitle if ($imgtitle);
  }
  else {
    $linktext =~ s/<[^>]+>/ /g;
    $linktext = 'linktext: '.&del_wsp (&trim ($linktext));
  }
  return $linktext;
}

sub domain {
  my $res = shift;
  $res =~ s/^(http:\/\/)?(www\.)?([^\/]+)(\/.*|)$/$3/;
  return $res;
}

sub del_wsp {
  my ($t, @a);
  foreach $t (@_) {
    $t =~ s/\s+/ /g;
    push (@a, $t);
  }
  return wantarray ? @a : $a[0];
}

sub trim {
  my ($t, @a);
  foreach $t (@_) {
    $t =~ s/^\s+//;
    $t =~ s/\s+$//;
    push (@a, $t);
  }
  return wantarray ? @a : $a[0];
}
